
// Project: webCDwriter - Network CD Writing
// File: Cdrecord.cpp
// Author: Jrg Haeger, 13.03.2000

#include "Config.h"
#include "Support.h"
#include "Log.h"
#include "Cdrecord.h"

Cdrecord::Cdrecord(const char *outName):
	Process(config.getCdrecordPath(), outName)
{
	for (int i = 0; i < 256; i++)
	{
		results[i].key = -1;
		results[i].code = 0;
		results[i].qual = 0;
		results[i].str = NULL;
	}
	lastCommand = lastError = -1;

	cannotReadTOC = cannotReadSessionOffset = 0;
	fixating = 0;

	// add dev-argument of cdrecord
	char str[4*1024];
	sprintf(str, "dev=%s", config.getCdrecordDevArg());
	addArg(str);
}

Cdrecord::~Cdrecord()
{
	for (int i = 0; i < 256; i++)
		if (results[i].str != NULL)
			delete results[i].str;
}

const char *Cdrecord::getLastError()
{
	if (lastError < 0)
		return NULL;
	else
	{
		static char str[30 + 1];
		strncpy(str, results[lastError].str, sizeof str - 1);
		str[sizeof str - 1] = 0;
		return str;
	}
}

int Cdrecord::isBlankDisk()
{
	// a blank disk is detected by read toc
	return results[0x43].code == 0x8F && results[0x43].qual == 0x00;
}

int Cdrecord::isInProcessOfBecomingReady()
{
	// logical unit is in process of becoming ready
	return results[0x00].code == 0x04 && results[0x00].qual == 0x01;
}

int Cdrecord::isMediumNotPresent()
{
	// medium not present
	return results[0x00].code != 0x00;
}

const char *Cdrecord::readLine()
{
	const char *lineStr = Process::readLine();
	if (lineStr == NULL)
		return NULL;

	// see cdrecord-1.8/libscg/scsierrs.c
	if (sscanf(lineStr, "CDB: %x", &lastCommand) == 1)
	{
		if (0 <= lastCommand && lastCommand < 256)
		{
			results[lastCommand].key = 0;
			results[lastCommand].code = 0;
			results[lastCommand].qual = 0;
			results[lastCommand].str = NULL;
		}
	}
	else
	if (sscanf(lineStr, "Sense Key: %x", &key) == 1)
		;
	else
	if (sscanf(lineStr, "Sense Code: %x Qual %x", &code, &qual) == 2)
	{
		if (0 <= lastCommand && lastCommand < 256)
		{
//			printf("cmd = %02x, key = %02x, "
//				"code = %02x, qual = %02x\n",
//				lastCommand, key, code, qual);
			lastError = lastCommand;
			results[lastCommand].key = key;
			results[lastCommand].code = code;
			results[lastCommand].qual = qual;

			char *a = strchr(lineStr, '(');
			char *b = strchr(lineStr, ')');
			if (a != NULL && b != NULL && a < b)
			{
				char str[30 + 1];
				int n = min(b - a - 1, sizeof str - 1);
				strncpy(str, a + 1, n);
				str[n] = 0;
				results[lastCommand].str = strDup(str);
			}
		}
	}
	else
	if (strstr(lineStr, "Cannot read TOC header") != NULL
		|| strstr(lineStr, "Cannot read TOC/PMA") != NULL)
		cannotReadTOC = 1;
	else
	if (strstr(lineStr, "Cannot read session offset") != NULL)
		cannotReadSessionOffset = 1;
	else
	if (strstr(lineStr, "Drive needs to reload") != NULL)
	{
		// the media to return to proper status.
		terminate();
		throw "reload media";
	}
	else
	if (strstr(lineStr, "Fixating...") == lineStr)
		fixating = 1;
	else
	if (strstr(lineStr, "may not fit") != NULL)
	{
		terminate();
		throw "may not fit";
	}

	return lineStr;
}

