
// Project: webCDwriter - Network CD Writing
//	Simulation of cdrecord for testing CDWserver

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

char lastMode[20];
int lastVal = 0;

void log(const char *mode)
{
	FILE *out = fopen("/tmp/cdrecord.last", "a");
	if (out == NULL)
		return;

	int val = 0;
	if (strcmp(mode, lastMode) == 0)
		val = lastVal + 1;
	fprintf(out, "%s %d\n", mode, val);
	fclose(out);
}

void printError(const char *arg0,
	int cmd, int key, int code, int qual, const char *str)
{
	printf("%s: Eingabe-/Ausgabefehler.\n", arg0);
	printf("CDB: %02x 00 00 00 00 00\n", cmd);
	printf("Sense Key: 0x%x Not Ready, Segment 0\n", key);
	printf("Sense Code: 0x%02x Qual 0x%02x (%s) Fru 0x0\n",
		code, qual, str);
}

int main(int argc, char *args[])
{
	char error[40] = "no";
	FILE *in = fopen("/tmp/cdrecord.config", "r");
	if (in != NULL)
	{
		while (1)
		{
			char lineStr[80];
			fgets(lineStr, sizeof lineStr, in);
			if (feof(in))
				break;
			sscanf(lineStr, "error=%s", error);
		}
		fclose(in);
	}

	in = fopen("/tmp/cdrecord.last", "r");
	if (in == NULL)
		strcpy(lastMode, "start");
	else
	{
		while (1)
		{
			char lineStr[80];
			fgets(lineStr, sizeof lineStr, in);
			if (feof(in))
				break;
			sscanf(lineStr, "%s %d", lastMode, &lastVal);
		}
		fclose(in);
	}

	int dev = 0;
	enum { eject, files, help, inq, load, msinfo, toc, version }
		mode = help;
	for (int i = 1; i < argc; i++)
		if (strncmp(args[i], "dev=", 4) == 0)
			dev = 1;
		else
		if (strcmp(args[i], "-eject") == 0)
			mode = eject;
		else
		if (strcmp(args[i], "-inq") == 0)
			mode = inq;
		else
		if (strcmp(args[i], "-load") == 0)
			mode = load;
		else
		if (strcmp(args[i], "-msinfo") == 0)
			mode = msinfo;
		else
		if (strcmp(args[i], "-toc") == 0)
			mode = toc;
		else
		if (strcmp(args[i], "-version") == 0)
			mode = version;
		else
		if (args[i][0] != '-')
			mode = files;

	if (mode != version && !dev)
		mode = help;

	switch (mode)
	{
		case eject:
		printf("ok\n");
		log("eject");
		return 0;

		case help:
		if (!dev)
			printf("%s: No CD/DVD-Recorder device specified.\n",
				args[0]);
		printf("%s: Usage: %s [options] track1...trackn\n",
			args[0], args[0]);
		log("help");
		return 0;

		case load:
		printf("ok\n");
		log("load");
		return 0;

		case msinfo:
		printf("blank\n");
		log("msinfo");
		return 0;

		case toc:
		printf("TOC\n");
		log("toc");
		return 0;

		case inq:
		if (strcmp(lastMode, "inq") != 0
			|| lastVal < 3)
			printError(args[0], 0x00, 0x2, 0x3A, 0x00,
				"medium not present");
		printf("Identifikation : 'dummy'\n");
		log("inq");
		return 0;

		case version:
		printf("CdrecordDummy 1.8 (i686-pc-linux-gnu)\n");
		log("version");
		return 0;
	}

	log("write");
	printf("CdrecordDummy 1.8 (i686-pc-linux-gnu)\n");
	if (strcmp(error, "reloadMedia") == 0)
		printf("%s: Drive needs to reload the media"
			" to return to proper status.\n", args[0]);
	else
	if (strcmp(error, "servoFailure") == 0)
	{
		printError(args[0], 0x5B, 0x4, 0x09, 0x01,
				"tracking servo failure");
		return 1;
	}

	in = fopen("/tmp/cdrecord.out", "r");
	assert(in != NULL);

	while (1)
	{
		char lineStr[1024];
		int i;
		for (i = 0; i < sizeof lineStr - 1; i++)
		{
			int ch = fgetc(in);
			if (ch == EOF)
				break;
			if (ch == '\r' || ch == '\n')
			{
				lineStr[i++] = ch;
				break;
			}
			lineStr[i] = ch;
		}
		lineStr[i] = 0;
		if (i == 0)
			break;

		printf("%s", lineStr);
		fflush(stdout);

		if (strstr(lineStr, "Track") == NULL)
			usleep(100000);
		else
			sleep(1);
	}
}

