
// Project: webCDwriter - Network CD Writing
// Author: Jrg Haeger
//	joerg.haeger@Uni-Bielefeld.de

import java.io.*;
import java.text.*;
import javax.swing.*;

public class BurnSession extends Command
{
	final static String
	hintCDdetected = "CD im Laufwerk",
	hintCDrecordStarted = "Brennvorgang begonnen",
	hintImageReady = "CD-Image fertig",
	hintMakingImage = "erzeuge ein CD-Image",
	hintWriterReserved = "Brenner reserviert"
	;

	QueueDialog queueDialog;
	JOptionPane pane;
	BurnDialog burnDialog;
	private boolean multiSession, offline, simulate;
	private int speed;
	int totalKB, oldFreeKB, newFreeKB;

	BurnSession(
		boolean multiSession,
		boolean offline,
		boolean simulate,
		int speed)
	{
		this.multiSession = multiSession;
		this.offline = offline;
		this.simulate = simulate;
		this.speed = speed;
		burnDialog = new BurnDialog();
	}

	void cancelCommand()
	{
		try {
			writeCommand("noop");
		}
		catch (java.io.IOException e)
		{
			System.out.println("cancelCommand: IOException");
		}
	}

	void parseCapacityHint(String lineStr) throws IOException
	{
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("capacity"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		totalKB = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		oldFreeKB = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		newFreeKB = (int)tokenizer.nval;
	}

	void parseCDrecordLine(String lineStr) throws IOException
	{
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));
		tokenizer.wordChars(':', ':');

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("Track"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int track = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_WORD)
			return;
		if (!tokenizer.sval.equals(":"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int n = (int)tokenizer.nval;

		if (tokenizer.nextToken() != StreamTokenizer.TT_WORD)
			return;
		if (!tokenizer.sval.equals("of"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int N = (int)tokenizer.nval;

		if (n == 0)
			burnDialog.setRange(0, N);
		burnDialog.setProgress(n);
	}

	void parseMkisofsHint(String lineStr) throws IOException
	{
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("mkisofsPercentDone"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		burnDialog.setProgress((int)tokenizer.nval);
	}

	void parseRankHint(String lineStr) throws IOException
	{
		StreamTokenizer tokenizer
			= new StreamTokenizer(
				new StringReader(lineStr));

		tokenizer.nextToken();
		if (!tokenizer.sval.equals("rank"))
			return;

		if (tokenizer.nextToken() != StreamTokenizer.TT_NUMBER)
			return;
		int rank = (int)tokenizer.nval;
		queueDialog.setRank(rank);
	}

	void processHint(String str)
	{
//		System.out.println("processHint " + str);
		try
		{
			if (str.startsWith("rank"))
				parseRankHint(str);
			else
			if (str.startsWith("mkisofsPercentDone"))
				parseMkisofsHint(str);
			else
			if (str.startsWith("Track"))
				parseCDrecordLine(str);
			else
			if (str.startsWith("capacity"))
				parseCapacityHint(str);
		}
		catch (IOException e)
		{
		}

		if (str.equals(hintWriterReserved))
		{
			hideDialog();
			pane = new JOptionPane(
				infoStr,
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null, cancelButton);
			JDialog dialog = pane.createDialog(
				CDcreator.mainWin, i18n("titleInsertCD"));
			showDialog(dialog);
		}
		if (str.equals(hintCDdetected))
		{
			burnDialog.setTitle(i18n("titleCreateImage"));
			burnDialog.setMessage(i18n("createImage"));
			burnDialog.setRange(0, 100);
			replaceDialog(burnDialog);
		}
		if (str.equals(hintCDrecordStarted))
		{
//			hideDialog();
			burnDialog.setTitle(i18n("titleBurn"));
			burnDialog.setMessage(i18n("burn"));
			burnDialog.setProgress(0);
//			showDialog(burnDialog);
		}
	}

	void processInfo(String str)
	{
		super.processInfo(str);
		if (pane != null)
			pane.setMessage(infoStr);
	}

	public void run()
	{
		queueDialog = new QueueDialog(
			CDcreator.mainWin, i18n("titleReserve"),
			i18n("reserve"));
		showDialog(queueDialog);

		execCommand("setSpeed " + speed);
		execCommand("setShowCdrecord");
		execCommand("setShowMkisofs");
		if (simulate)
			execCommand("setLaserOff");

		String str = "burnSession";
		if (multiSession)
			str = str.concat(" multi");
		if (offline)
			str = str.concat(" offline");
		execCommand(str);

		hideDialog();

		if (offline)
		{
			JOptionPane pane = new JOptionPane(
				infoStr + "\n"
				+ Version.product + " "
				+ i18n("willNowExit"),
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null, okButton);
			JDialog dialog = pane.createDialog(
				CDcreator.mainWin, i18n("titleInsertCD"));
			showInfoWin(dialog);
			super.run();
			return;
		}

		MessageFormat formatter = new MessageFormat("");
		formatter.setLocale(CDcreator.locale);
		formatter.applyPattern(i18n("burnSuccess"));
		Object[] args = {
			new Integer((oldFreeKB-newFreeKB) / 1024),
			new Integer(newFreeKB / 1024),
			new Integer(totalKB / 1024) };

		JDialog dialog;
		if (!lastResultOK())
			dialog = new JOptionPane(
				lastResult,
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null, okButton).createDialog(
				CDcreator.mainWin, i18n("error"));
		else
		dialog = new JOptionPane(
			formatter.format(args),
			JOptionPane.INFORMATION_MESSAGE,
			JOptionPane.DEFAULT_OPTION,
			null, okButton).createDialog(
			CDcreator.mainWin, i18n("titleBurnResult"));

		try {
			if (cancel)
				readResult();
		}
		catch (java.io.IOException e)
		{
			System.out.println("BurnSession: IOException");
		}

		execCommand("closeSession");
		if (!cancel)
			showInfoWin(dialog);

		super.run();
	}
}

